@@ -22,11 +22,14 @@ class Models:
 
     _FILES = ["model.bin", "config.json", "vocabulary.txt", "tokenizer.json"]
 
-    def __init__(self):
-        default = os.path.join(os.path.expanduser("~"), ".cache")
-        self.download_root = os.path.join(
-            os.getenv("XDG_CACHE_HOME", default), "whisper-ctranslate2"
-        )
+    def __init__(self, cache_directory=None):
+        if cache_directory:
+            self.download_root = cache_directory
+        else:
+            default = os.path.join(os.path.expanduser("~"), ".cache")
+            self.download_root = os.path.join(
+                os.getenv("XDG_CACHE_HOME", default), "whisper-ctranslate2"
+            )
 
     def get_list(self) -> str:
         return list(self._MODELS.keys())
@@ -76,7 +79,6 @@ class Models:
         model_path = self.get_model_path(model)
         os.makedirs(model_path, exist_ok=True)
 
-        expected_sha256 = url.split("/")[-2].rsplit(".", 1)[0]
         download_target = os.path.join(model_path, os.path.basename(url))
 
         if os.path.exists(download_target) and not os.path.isfile(download_target):
