@@ -3,6 +3,8 @@ import os
 from .transcribe import Transcribe, TranscriptionOptions
 from .models import Models
 from .languages import LANGUAGES, TO_LANGUAGE_CODE
+import numpy as np
+import warnings
 
 
 def optional_int(string):
@@ -21,11 +23,6 @@ def optional_float(string):
     return None if string == "None" else float(string)
 
 
-def compression_ratio(text) -> float:
-    text_bytes = text.encode("utf-8")
-    return len(text_bytes) / len(zlib.compress(text_bytes))
-
-
 def read_command_line():
     parser = argparse.ArgumentParser(
         formatter_class=argparse.ArgumentDefaultsHelpFormatter
@@ -39,6 +36,12 @@ def read_command_line():
         choices=Models().get_list(),
         help="name of the Whisper model to use",
     )
+    parser.add_argument(
+        "--model_dir",
+        type=str,
+        default=None,
+        help="the path to save model files; uses ~/.cache/whisper-ctranslate2 by default",
+    )
     parser.add_argument(
         "--output_dir",
         "-o",
@@ -84,10 +87,17 @@ def read_command_line():
         help="number of threads used for CPU inference",
     )
 
-    #    parser.add_argument("--model_dir", type=str, default=None, help="the path to save model files; uses ~/.cache/whisper by default")
     parser.add_argument(
         "--temperature", type=float, default=0, help="temperature to use for sampling"
     )
+
+    parser.add_argument(
+        "--temperature_increment_on_fallback",
+        type=optional_float,
+        default=0.2,
+        help="temperature to increase when falling back when the decoding fails to meet either of the thresholds below",
+    )
+
     parser.add_argument(
         "--best_of",
         type=optional_int,
@@ -131,7 +141,6 @@ def read_command_line():
         default=True,
         help="if True, provide the previous output of the model as a prompt for the next window; disabling may make the text inconsistent across windows, but the model becomes less prone to getting stuck in a failure loop",
     )
-    #   parser.add_argument("--temperature_increment_on_fallback", type=optional_float, default=0.2, help="temperature to increase when falling back when the decoding fails to meet either of the thresholds below")
     parser.add_argument(
         "--compression_ratio_threshold",
         type=optional_float,
@@ -165,14 +174,38 @@ def read_command_line():
     # CTranslate2 specific parameters
     parser.add_argument(
         "--compute_type",
-        choices=["default", "auto", "int8", "int8_float16", "int16", "float16", "float32"],
+        choices=[
+            "default",
+            "auto",
+            "int8",
+            "int8_float16",
+            "int16",
+            "float16",
+            "float32",
+        ],
         default="auto",
         help="Type of quantization to use (see https://opennmt.net/CTranslate2/quantization.html)",
     )
+    parser.add_argument(
+        "--model_directory",
+        type=str,
+        default=None,
+        help="Directory where to find a CTranslate Whisper model (e.g. fine-tuned model)",
+    )
 
     return parser.parse_args().__dict__
 
 
+def _from_language_to_iso_code(language):
+    if language is not None:
+        language_name = language.lower()
+        if language_name not in LANGUAGES:
+            if language_name in TO_LANGUAGE_CODE:
+                language = TO_LANGUAGE_CODE[language_name]
+
+    return language
+
+
 def main():
     args = read_command_line()
     output_dir: str = args.pop("output_dir")
@@ -185,6 +218,27 @@ def main():
     device: str = args.pop("device")
     compute_type: str = args.pop("compute_type")
     verbose: bool = args.pop("verbose")
+    model_directory: str = args.pop("model_directory")
+    cache_directory: str = args.pop("model_dir")
+
+    temperature = args.pop("temperature")
+    if (increment := args.pop("temperature_increment_on_fallback")) is not None:
+        temperature = tuple(np.arange(temperature, 1.0 + 1e-6, increment))
+    else:
+        temperature = [temperature]
+
+    language = _from_language_to_iso_code(language)
+
+    if (
+        not model_directory
+        and model.endswith(".en")
+        and language not in {"en", "English"}
+    ):
+        if language is not None:
+            warnings.warn(
+                f"{model} is an English-only model but receipted '{language}'; using English instead."
+            )
+        language = "en"
 
     options = TranscriptionOptions(
         beam_size=args.pop("beam_size"),
@@ -195,7 +249,7 @@ def main():
         no_speech_threshold=args.pop("no_speech_threshold"),
         compression_ratio_threshold=args.pop("compression_ratio_threshold"),
         condition_on_previous_text=args.pop("condition_on_previous_text"),
-        temperatures=[args.pop("temperature")],
+        temperature=temperature,
         initial_prompt=args.pop("initial_prompt"),
         #        prefix = None,
         #        suppress_blank = None,
@@ -207,7 +261,13 @@ def main():
         #        append_punctuations = None
     )
 
-    model_dir = Models().get_model_dir(model)
+    if model_directory:
+        model_filename = os.path.join(model_directory, "model.bin")
+        if not os.path.exists(model_filename):
+            raise RuntimeError(f"Model file '{model_filename}' does not exists")
+        model_dir = model_directory
+    else:
+        model_dir = Models(cache_directory).get_model_dir(model)
 
     for audio_path in args.pop("audio"):
         Transcribe().inference(
